using System.Net.Mime;
using System.Text.RegularExpressions;
using System.Xml;
using FitConnect.Interfaces;
using FitConnect.Models.Api.Metadata;
using FitConnect.Models.Api.Set;
using Newtonsoft.Json;
using Attachment = FitConnect.Models.Attachment;

namespace FitConnect;

public class SubmissionBuilder : SendableSubmission, ISubmissionWithDestination,
    ISubmissionWithServiceType, ISubmissionWithData, ISubmissionWithReplyChannel,
    ISubmissionWithPaymentInformation {
    internal SubmissionBuilder() {
    }

    public ISubmissionWithData AddAttachment(Attachment attachment) => AddAttachments(attachment);

    public ISubmissionWithData AddAttachments(List<Attachment> attachments) {
        return AddAttachments(attachments.ToArray());
    }

    public ISubmissionWithData SetPaymentInformation(PaymentInformation paymentInformation) {
        PaymentInformation = paymentInformation;
        return this;
    }

    public ISubmissionWithData SetReplyChannel(IReplyChannelType replyChannel) {
        ReplyChannel = ReplyChannel.FromType(replyChannel);
        return this;
    }

    public ISubmissionWithData SetAuthenticationInformation(
        params AuthenticationInformation[] authenticationInformation) {
        AuthenticationInformation = authenticationInformation;
        return this;
    }

    private ISubmissionWithData AddAttachments(string fileName, string description,
        string mimeType,
        FileStream fileStream) {
        byte[]? buffer;
        using (fileStream) {
            buffer = new byte[fileStream.Length];
            fileStream.Read(buffer, 0, (int)fileStream.Length);
        }

        var attachment = Attachment.FromByteArray(buffer, mimeType, fileName, description);

        return AddAttachments(attachment);
    }

    public ISubmissionWithData AddAttachments(params Attachment[] attachments) {
        Attachments ??= new List<Attachment>();

        foreach (var attachment in attachments) {
            Attachments.Add(attachment);
        }

        return this;
    }

    public SendableSubmission Build() {
        return this;
    }

    public ISubmissionWithServiceType SetServiceType(string serviceIdentifier, string serviceName) {
        if (string.IsNullOrWhiteSpace(serviceIdentifier) || !Regex.IsMatch(serviceIdentifier,
                FitConnectClient.ServiceIdentifierPattern))
            throw new ArgumentException("Invalid leika key");

        ServiceName = serviceName;
        ServiceIdentifier = serviceIdentifier;
        return this;
    }

    public ISubmissionWithData SetXmlData(string data) {
        try {
            var doc = new XmlDocument();
            doc.LoadXml(data);
        }
        catch (Exception e) {
            throw new ArgumentException("The data must be valid XML string", e);
        }

        Data = data;
        DataMimeType = MediaTypeNames.Application.Xml;
        return this;
    }


    public ISubmissionWithData SetJsonData(string data) {
        try {
            JsonConvert.DeserializeObject(data);
        }
        catch (Exception e) {
            throw new ArgumentException("The data must be valid JSON string", e);
        }

        Data = data;
        DataMimeType = MediaTypeNames.Application.Json;
        return this;
    }

    /// <summary>
    /// Setting the destination id for this submission.
    /// </summary>
    /// <remarks>This method overwrites a previously set value</remarks>
    /// <param name="destinationId">Guid for the destination the submission is meant for</param>
    /// <returns></returns>
    public ISubmissionWithDestination SetDestination(Guid destinationId) {
        this.DestinationId = destinationId;
        return this;
    }
}

public interface ISubmissionWithDestination {
    /// <summary>
    /// Setting the service identifier and service name for this submission.
    /// </summary>
    /// <remarks>This method overwrites a previously set value</remarks>
    /// <param name="serviceIdentifier">Service identifier urn for the submission</param>
    /// <param name="serviceName">An optional name for the Service</param>
    /// <returns></returns>
    ISubmissionWithServiceType SetServiceType(string serviceIdentifier,
        string serviceName = "Service");
}

public interface ISubmissionWithServiceType {
    /// <summary>
    /// Setting the data for this submission as JSON.
    /// </summary>
    /// <param name="data">String containing JSON formatted data</param>
    /// <returns></returns>
    ISubmissionWithData SetJsonData(string data);

    /// <summary>
    /// Setting the data for this submission as XML.
    /// </summary>
    /// <param name="data">String containing XML formatted data</param>
    /// <returns></returns>
    ISubmissionWithData SetXmlData(string data);
}

public interface ISubmissionWithData : ISubmissionReadyToBuild {
    /// <summary>
    /// Adding attachments to this submission.
    /// </summary>
    /// <param name="attachments">Single or multiple attachment object</param>
    /// <returns></returns>
    ISubmissionWithData AddAttachments(params Attachment[] attachments);

    /// <summary>
    /// Adding an attachment to this submission.
    /// </summary>
    /// <param name="attachment">An instance of an attachment</param>
    /// <returns></returns>
    ISubmissionWithData AddAttachment(Attachment attachment);

    /// <summary>
    /// Adding attachments to this submission.
    /// </summary>
    /// <param name="attachments">List of attachments</param>
    /// <returns></returns>
    ISubmissionWithData AddAttachments(List<Attachment> attachments);

    /// <summary>
    /// Setting the payment information for this submission.
    /// </summary>
    /// <remarks>This method overwrites a previously set value</remarks>
    /// <param name="paymentInformation">Instance of the <see cref="PaymentInformation"/></param>
    /// <returns></returns>
    ISubmissionWithData SetPaymentInformation(PaymentInformation paymentInformation);

    /// <summary>
    /// Setting an optional reply channel for this submission.
    /// </summary>
    /// <remarks>This method overwrites a previously set value</remarks>
    /// <param name="replyChannel">The reply channel that has to be used for a communication to the user</param>
    /// <returns></returns>
    ISubmissionWithData SetReplyChannel(IReplyChannelType replyChannel);

    /// <summary>
    /// Adding the proof for identification
    /// </summary>
    /// <remarks>This method overwrites a previously set value</remarks>
    /// <param name="authenticationInformation">Instance of the <see cref="AuthenticationInformation"/></param>
    /// <returns></returns>
    ISubmissionWithData SetAuthenticationInformation(
        params AuthenticationInformation[] authenticationInformation);
}

public interface ISubmissionWithPaymentInformation : ISubmissionReadyToBuild {
    ISubmissionWithData SetPaymentInformation(PaymentInformation paymentInformation);
}

public interface ISubmissionWithReplyChannel : ISubmissionReadyToBuild {
    /// <summary>
    /// 
    /// </summary>
    /// <param name="replyChannel"></param>
    /// <returns></returns>
    ISubmissionWithData SetReplyChannel(IReplyChannelType replyChannel);
}

public interface ISubmissionReadyToBuild {
    /// <summary>
    /// Build the submission to create a sendable object
    /// </summary>
    /// <returns>Instance of <see cref="SendableSubmission"/> that can be sent via <see cref="ISender.SendAsync(FitConnect.SendableSubmission)"/></returns>
    SendableSubmission Build();
}
