using FitConnect.Models.Api.Metadata;
using FitConnect.Services.Models;
using FitConnect.Services.Models.v1.Submission;
using Newtonsoft.Json;

namespace FitConnect.Models;

public class Submission : SendableSubmission, ISubmissionBody {
    internal Submission() {
    }

    internal Guid Id { get; set; } = Guid.Empty;
    public Destination Destination { get; internal set; } = new();

    // public List<Attachment>? Attachments { get; internal set; } = new();
    public List<Guid> AttachmentIds { get; init; } = new();

    public ServiceType ServiceType { get; internal set; } = new();

    public Callback? Callback { get; init; }

    public MetadataDto? Metadata { get; internal set; }

    // public string? Data { get; internal set; }
    public string? EncryptedMetadata { get; internal set; }
    public string? EncryptedData { get; internal set; }
    public string? MetaAuthentication => EncryptedMetadata?.Split('.').Last();

    public string? DataAuthentication => EncryptedData?.Split('.').Last();
    // public string? DataMimeType { get; internal set; }

    public Guid SubmissionId => Id;
    public Guid CaseId { get; internal set; } = Guid.Empty;
    internal string? GeneratedMetadata { get;  set; }


    // public string DestinationId {
    //     get => Destination.DestinationId ?? throw new ArgumentNullException(nameof(DestinationId));
    //     set => Destination.DestinationId = value;
    // }
    //

    public bool IsSubmissionReadyToAdd(out string? error) {
        var innerError = "";
        if (string.IsNullOrEmpty($"{DestinationId}")) innerError += "DestinationId is required\r\n";

        if (ServiceType.IsValid()) innerError += "ServiceType is invalid\r\n";

        if (string.IsNullOrWhiteSpace(innerError)) {
            error = null;
            return true;
        }

        error = innerError.Trim();
        return false;
    }

    public bool IsSubmissionReadyToSend() {
        return true;
    }

    public static implicit operator SubmissionForPickup(Submission sub) {
        return new SubmissionForPickup {
            Id = sub.Id,
            CaseId = sub.CaseId,
            DestinationId = sub.DestinationId
        };
    }

    public static explicit operator Submission(SubmissionForPickup dto) {
        return new Submission {
            Id = dto.Id,
            Callback = null,
            DestinationId = dto.DestinationId,
            ServiceType = new ServiceType()
        };
    }

    public static explicit operator Submission(SubmissionDto dto) {
        return new Submission {
            Id = dto.SubmissionId,
            Callback = dto.Callback == null ? null : (Callback)dto.Callback,
            DestinationId = dto.DestinationId ??
                            throw new NullReferenceException(nameof(dto.DestinationId)),
            ServiceType =
                dto.ServiceType == null ? new ServiceType() : (ServiceType)dto.ServiceType,
            EncryptedData = dto.EncryptedData,
            EncryptedMetadata = dto.EncryptedMetadata,
            AttachmentIds = dto.AttachmentIds ?? new List<Guid>(),
            CaseId = dto.CaseId
        };
    }

    public static explicit operator SubmitSubmissionDto(Submission dto) {
        return new SubmitSubmissionDto {
            EncryptedData = dto.EncryptedData,
            EncryptedMetadata = dto.EncryptedMetadata
        };
    }

    public static explicit operator CreateSubmissionDto(Submission sub) {
        var result = new CreateSubmissionDto {
            AnnouncedAttachments = sub.Attachments?.Select(a => a.Id).ToList(),
            DestinationId = sub.DestinationId
        };
        result.ServiceType = (ServiceTypeDto)sub.ServiceType;
        if (sub.Callback != null) result.Callback = (CallbackDto)sub.Callback;

        return result;
    }
}

public interface ISubmissionBody {
    public Guid SubmissionId { get; }
    public Guid CaseId { get; }
    public Guid DestinationId { get; }
}

public class SentSubmission : ISubmissionBody {
    public SentSubmission(Guid submissionId, Guid caseId, Guid destinationId,
        AuthenticationTags authenticationTags) {
        SubmissionId = submissionId;
        CaseId = caseId;
        DestinationId = destinationId;
        AuthenticationTags = authenticationTags;
    }

    public Guid SubmissionId { get; internal set; }
    public Guid CaseId { get; internal set; }
    public Guid DestinationId { get; internal set; }
    public AuthenticationTags AuthenticationTags { get; internal set; }


    public static explicit operator SentSubmission(Submission submission)
        => new SentSubmission(
            submission.Id,
            submission.CaseId,
            submission.DestinationId,
            new AuthenticationTags {
                Data = submission.DataAuthentication,
                Metadata = submission.MetaAuthentication,
                Attachments =
                    submission.Attachments?.ToDictionary(i => i.Id,
                        i => i.AttachmentAuthentication!) ?? new Dictionary<Guid, string>()
            });
}

public class SubmissionBody : ISubmissionBody {
    public SubmissionBody() {
    }

    private SubmissionBody(Guid submissionId, Guid caseId, Guid destinationId) {
        CaseId = caseId;
        SubmissionId = submissionId;
        DestinationId = destinationId;
    }

    [JsonProperty("submissionId")]
    public Guid SubmissionId { get; init; }

    [JsonProperty("caseId")]
    public Guid CaseId { get; init; }

    [JsonProperty("destinationId")]
    public Guid DestinationId { get; init; }

    public static implicit operator SubmissionBody(Submission submission) {
        return new SubmissionBody(
            submission.Id, submission.CaseId, submission.DestinationId);
    }
}
