using FitConnect;
using FitConnect.Encryption;
using FitConnect.Models;
using FitConnect.Models.v1.Api;
using FluentAssertions;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;

namespace E2ETest;

[Order(30)]
[TestFixture]
public class StraightPreEncryptedForwardTest : EndToEndTestBase {
    private SentSubmission _submission = null!;

    [Order(10)]
    [Test]
    public void SendingSubmission() {
        // Arrange
        var publicKey = (Sender as Sender)!.GetPublicKeyForDestinationAsync(Settings.DestinationId)
            .Result;

        var joseEncryptor = new JoseEncryptor();
        var encryptedAttachment =
            joseEncryptor.Encrypt(publicKey, File.ReadAllBytes("Test.pdf"));

        var data = JsonConvert.SerializeObject(new {
            Name = "Encrypted Test",
            Content = "A simple test to upload pre-encrypted data"
        });

        var dataHash = FitEncryption.CalculateHash(data);
        var encryptedData = joseEncryptor.Encrypt(publicKey, data);

        var attachment = Attachment.FromPath("Test.pdf",
            System.Net.Mime.MediaTypeNames.Application.Pdf, description: "Simple Test",
            fileName: "Test.pdf");
        var metadata = FitConnect.Sender.CreateMetadata(new Submission {
            DataMimeType = "application/json",
            Data = encryptedData,
            Attachments = new List<Attachment> { attachment },
            ServiceType = new ServiceType
                { Identifier = Settings.ServiceIdentifier, Name = "Straight forward test" }
        }, dataHash);

        var encryptedMetadata = joseEncryptor.Encrypt(publicKey, metadata);


        // Act
        var submission = SendableEncryptedSubmission.Builder()
            .SetDestination(Settings.DestinationId)
            .SetServiceType(Settings.ServiceIdentifier, "Straight forward test")
            .SetEncryptedMetadata(encryptedMetadata)
            .SetEncryptedData(encryptedData)
            .AddEncryptedAttachment(attachment.Id, encryptedAttachment)
            .Build();

        var result = Sender.SendAsync(submission).Result;

        // Assert   
        _submission = result;

        _submission.AuthenticationTags.Attachments.Should().NotBeNullOrEmpty();
        _submission.AuthenticationTags.Attachments!.All(item =>
                !(string.IsNullOrWhiteSpace($"{item.Key}") ||
                  string.IsNullOrWhiteSpace(item.Value)))
            .Should().BeTrue();
    }

    [Test]
    [Order(20)]
    public void Sender_GetEventLogForSubmission() {
        // Act
        var status = Sender.GetEventLogAsync(_submission.CaseId, Settings.DestinationId).Result;

        // Assert
        status.Count.Should().BeGreaterThan(0);
        status.Any(s => s.EventType == EventType.Submit).Should().BeTrue();
        status.ForEach(
            s => Logger.LogInformation("Status {When} {Event}", s.EventTime, s.EventType));
    }

    [Test]
    [Order(21)]
    public void Sender_GetStatusForSubmission() {
        var (status, problems) = Sender.GetStatusForSubmissionAsync(_submission).Result;
        status.Should().Be(EventType.Submit);
    }


    [Test]
    [Order(30)]
    public void Subscriber_GetSubmissionState() {
        // Act
        var status = Subscriber.GetEventLogAsync(_submission.CaseId, Settings.DestinationId).Result;

        // Assert
        status.Count.Should().BeGreaterThan(0);
        status.ForEach(
            s => Logger.LogInformation("Status {When} {Event}", s.EventTime, s.EventType));
    }

    [Test]
    [Order(40)]
    public void RequestSubmission() {
        var subscriberWithSubmission =
            Subscriber.RequestSubmissionAsync(_submission.SubmissionId).Result;

        var data = subscriberWithSubmission.GetData();
        Logger.LogInformation("Data {Data}", data);

        data.Should()
            .Contain("A simple test to upload pre-encrypted data");

        var attachments = subscriberWithSubmission.GetAttachmentsAsync().Result;
        attachments.First().Filename.Should().Be("Test.pdf");

        subscriberWithSubmission
            .AcceptSubmissionAsync(new Problems(Problems.ProblemTypeEnum.MissingSchema,
                "Schema missing")).Wait();
    }

    [Test]
    [Order(50)]
    public void Sender_GetEventLog_AfterAccepting() {
        // Act
        var status = Sender.GetStatusForSubmissionAsync(_submission).Result;

        // Assert
        var acceptanceMessage = status;
        acceptanceMessage.SubmissionState.Should().Be(EventType.Accept);

        acceptanceMessage.Problems.Should().NotBeEmpty();

        acceptanceMessage.Problems?.Any(p => p.Detail == "Schema missing").Should().BeTrue();
    }

    [Test]
    [Order(51)]
    public void Sender_GetStatusForSubmission_AfterAccepting() {
        // Act
        var status = Sender.GetStatusForSubmissionAsync(_submission).Result;

        // Assert
        var acceptanceMessage = status;
        acceptanceMessage.SubmissionState.Should().Be(EventType.Accept);

        acceptanceMessage.Problems?
            .Any(p => p.Detail == "Schema missing").Should().BeTrue();
    }

    [Test]
    [Order(52)]
    public void Sender_GetStatusForSubmission_FailsOnManipulatedAuthenticationTags() {
        var manipulated = new SentSubmission(_submission.SubmissionId, _submission.CaseId,
            _submission.DestinationId,
            new AuthenticationTags() {
                Data = _submission.AuthenticationTags.Data + "Invalid",
                Metadata = _submission.AuthenticationTags.Metadata + "Invalid",
                Attachments = _submission.AuthenticationTags.Attachments!.ToDictionary(
                    i => i.Key,
                    i => "InvalidTag!")
            });

        var exception = Assert.Throws<AggregateException>(() =>
            Sender.GetStatusForSubmissionAsync(manipulated).Wait());

        exception!.InnerExceptions.Any(e => e is AuthenticationTagException).Should().BeTrue();
    }
}
